#========================================================================================
# package      TestAddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  class for unit test realted to AddressValidation.pm
#========================================================================================
package TestAddressValidation;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::AddressValidation;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::MockObject;
use Test::Exception;

sub Main {
    testAddressObjectIsNotValid();
    testErrorCases();
    testIfTheAddressIsComplete();
    done_testing();
}
sub testAddressObjectIsNotValid {
    my $subTestName = (caller(0))[3];
    my $ShopAddress = undef;
    ## no critic ( ProhibitComplexRegexes )
    throws_ok( sub {
                        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress );
                   },
                   qr/PARAMETER_MISSING: Missing parameter AddressValidation needs valid address object in call to DE_EPAGES::DHLIntraship::API::AddressValidation::new/ms,
                   "$subTestName - is ShopAddress is a undefined"
    );
    $ShopAddress = _createMock_WrongObject();
    throws_ok( sub {
                        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress );
                   },
                   qr/WRONG_CLASS: AddressValidation needs valid address object is not an instance of DE_EPAGES::Address::API::Object::Address/ms,
                   "$subTestName - is ShopAddress is a blessed object but not an AddressObject"
    );
    ## use critic ( ProhibitComplexRegexes )
    $ShopAddress = _createMock_AddressObject();
    isa_ok (
        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress ),
        'DE_EPAGES::DHLIntraship::API::AddressValidation',
        "$subTestName - If the constuctor is called correctly it returns AddressValidation object"
    );
    return;
}

sub testErrorCases{
    my $subTestName = (caller(0))[3];
    my $BillingAddress = _createMock_AddressObject( {
            'Alias' => 'BillingAddress ',
            'Country' => {'Code2'=> 'XX'},
            'FirstName' => '',
            'MiddleName' => '',
            'LastName' => 'Doe',
            'FullName' => '123456789_123456789_123456789_123456789_123456789_1',
            'StreetOnly' => 'Pilatuspool',
            'HouseNumber' => 666,
            'Zipcode' => 12345,
            'City' => 'Hamburg',
            'JobTitle' => '',
            'Company' => '',
            'Salutation' => '',
            'Phone' => '',
            'Fax' => '',
            'PhoneCell' => '',
            'EMail'  => '',
            'ExtraAttribute'     => 'ExtraAttributeValue',
            'ExtraAttributeMandatory'    => 'ExtraAttributeValueMandatory',
        } );
    my $ExtraAttributeDefinition = {
            'LastName' => {'Mandatory'=> 1,'MinValue'=> 10, 'MaxValue'=>50},
    };
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress,$ExtraAttributeDefinition );
    is($AddressValidation->isComplete(),0,"$subTestName - is Address uncomplete - FirstName is missing");
    is($AddressValidation->{'FirstName'},'', "$subTestName - check a undefind mandatory Attribute (FirstName)- undef");
    is($AddressValidation->{'LastName'},'Doe', "$subTestName - check a definded mandatory Attribute - Doe");
    my $aWarningStates = $AddressValidation->getStatesAsWarning();
    is (scalar @{$aWarningStates},3,"$subTestName - their are 3 Warnings - 3");
    is ( @{$aWarningStates}[0]->code(),
        'FirstName is mandatory but not set',
        "$subTestName - First message points to the Missing Firstnames"
    );
    is ( @{$aWarningStates}[1]->code(),
        'LastName is too Short',
        "$subTestName - Second message points to the too short LastName"
    );
    is ( @{$aWarningStates}[2]->code(),
        'FullName is too Long',
        "$subTestName - Second message points to the too long FullName"
    );
}
sub testIfTheAddressIsComplete {
    my $subTestName = (caller(0))[3];
    ## no critic ( RequireNumberSeparators)
    my $BillingAddress = _createMock_AddressObject( {
            'Alias' => 'BillingAddress ',
            'Country' => {'Code2'=> 'DE'},
            'FirstName' => 'Jon',
            'MiddleName' => 'MiddleNameText',
            'LastName' => 'Doe',
            'FullName' => 'FullNameText',
            'StreetOnly' => 'Pilatuspool',
            'HouseNumber' => 666,
            'Zipcode' => 12345,
            'City' => 'Hamburg',
            'JobTitle' => '',
            'Company' => '',
            'Salutation' => '',
            'Phone' => '',
            'Fax' => '',
            'PhoneCell' => '',
            'EMail'  => '',
            'ExtraAttribute'     => 'ExtraAttributeValue',
            'ExtraAttributeMandatory'    => 'ExtraAttributeValueMandatory',
        } );
    ## use critic (RequireNumberSeparators)
    my $ExtraAttributeDefinition = {
            'ExtraAttribute' => {'Mandatory'=> 0,'MinValue'=> 1, 'MaxValue'=>50},
            'ExtraAttributeMandatory' => {'Mandatory'=> 1,'MinValue'=> 1, 'MaxValue'=>50},
    } ;
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new($BillingAddress, $ExtraAttributeDefinition);
    is($AddressValidation->isComplete(),1,"$subTestName - is Address complete - true");
    is($AddressValidation->{'FirstName'},'Jon', "$subTestName - check a mandatory Attribute - Jon");
    is($AddressValidation->{'MiddleName'},'MiddleNameText', "$subTestName - check a not mandatory Attribute - MiddleName");
    is($AddressValidation->{'Zipcode'},12345, "$subTestName - result for german zipcode - 12345"); ## no critic ( RequireNumberSeparators ProhibitMagicNumbers)
    is($AddressValidation->{'ExtraAttribute'},'ExtraAttributeValue', "$subTestName - result for extra Attribute - ExtraAttributeValue");
    is($AddressValidation->{'ExtraAttributeMandatory'},'ExtraAttributeValueMandatory', "$subTestName - result for Extra  mandatory Attribute - ExtraAttributeValueMandatory");
}


sub _createMock_AddressObject {
    my ($hAtributes) = @_;
    my $AddressObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hAtributes);
    return $AddressObject;
}

sub _createMock_WrongObject {
    my ($hAtributes) = @_;
    my $ObjectObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Object',$hAtributes);
    return $ObjectObject;
}

RunScript( Sub => \&Main );