#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::Builder
# state        private
#----------------------------------------------------------------------------------------
# description  test the SOAP Builder
#========================================================================================
package TestDHLIntrashipBuilder;
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHLIntraship::API::SOAP::Builder;
use Data::Dumper;
use Test::More;
use Test::Deep;

## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProtectPrivateVars)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # tests
            {
                local *DE_EPAGES::DHLIntraship::API::Services::_getOptions = sub{
                    return {};
                };
                my $DHLBuilder = DE_EPAGES::DHLIntraship::API::SOAP::Builder->new();

                TestPackInternational($DHLBuilder);
                TestGroupOther($DHLBuilder);
                TestGroupIdent($DHLBuilder);
                TestGroupDHLPaket($DHLBuilder);
                TestCreateShipmentDD($DHLBuilder);
                done_testing();
            }
        }
    );
}

sub MockObjectPM {
    {
        package DE_EPAGES::Address::API::Object::Address;
        sub id { return 'Alias'; }
        sub deleteChildren { return; };
        sub folder {my $self = shift; return $self; };
        sub children {my $self = shift; return $self->{'Folder'}; };
        sub alias {my $self = shift; return $self->{'Alias'} //'Alias';};
        sub insertChild {
            my $self = shift;
            my ($Class, $hInfo) = @_;
            my $obj = {};
            $obj->{'Class'}         = $Class;
            $obj->{'Alias'}         = $hInfo->{'ServiceName'};
            $obj->{'ServiceGroup'}  = $hInfo->{'ServiceGroup'};
            $obj->{'ServiceAmount'} = $hInfo->{'ServiceAmount'};
            bless $obj, 'DE_EPAGES::Address::API::Object::Address';
            push(@{$self->{'Folder'}}, $obj);
        };
        sub get {
            my $self = shift;
            my ($aNames) = @_;
            if(ref($aNames) ne 'ARRAY'){
                return $self->_get($aNames);
            }

            my $hInfo = {};
            foreach my $Name (@{$aNames}) {
                $hInfo->{$Name} = $self->_get($Name);
            }

            return $hInfo;
        }
        sub _get{
            my $self = shift;
            my ($name) = @_;
            if ($name ~~ [qw(LineItemContainer ShopAddress Address Country ShippingFulfillmentMethod)]) {
                return $self;
            }
            elsif ($name  eq 'EstimatedDeliveryDate' )
            {
                my $dt = DateTime->new(
                year       => 2222,
                month      => 2,
                day        => 22,
                hour       => 22,
                minute     => 22,
                );
                return $dt;
            }
            elsif ($name  eq 'CustomerNumber' )
            {
                return '01234567895302';
            }
            else
            {
                return $name;
            }
        }
        sub findChild {
            my $self = shift;
            my ($alias) = @_;
            my ($returnValue) =  grep {$_->{'Alias'} eq $alias}  @$self;
            return $returnValue;

        }
        return bless {'Folder' => [], 'Code2' => 'Code2' }, 'DE_EPAGES::Address::API::Object::Address';
    }
}

sub TestCreateShipmentDD {
    my ($DHLBuilder) = @_;
    my $testClass = MockObjectPM();

    note((caller(0))[3]);
    ## no critic (ProtectPrivateSubs)
    # it have to be overwritten because without this it will take the data from epages.conf.
    # unfornatunaly we cant inject this.
    * DE_EPAGES::DHLIntraship::API::SOAP::Builder::_assembleDeveloperAuthentification = sub { return undef; };
    ## use critic (ProtectPrivateSubs)
    my $testValue = $DHLBuilder->_createShipmentDD($testClass);
    is_deeply($testValue,
        {
          'Version' => {
                         'minorRelease' => 0,
                         'majorRelease' => 1,
                         'build' => 14
                       },
          'ShipmentOrder' => {
                               'Shipment' => {
                                               'ShipmentDetails' => {
                                                                      'Attendance' => {
                                                                                        'partnerID' => '02'
                                                                                      },
                                                                      'Notification' => [
                                                                                          {
                                                                                            'RecipientName' => 'LastName',
                                                                                            'RecipientEmailAddress' => 'EMail'
                                                                                          }
                                                                                        ],
                                                                      'EKP' => '0123456789',
                                                                      'ProductCode' => 'BPI',
                                                                      'ShipmentDate' => '2222-02-22',
                                                                      'DeclaredValueOfGoods' => 'LineItemsSum',
                                                                      'DeclaredValueOfGoodsCurrency' => 'EUR',,
                                                                      'CustomerReference' => 'Alias',
                                                                      'ShipmentItem' => {
                                                                                          'WeightInKG' => 'TotalWeight',
                                                                                          'ParcelWidth' => '0',
                                                                                          'ParcelLength' => '0',
                                                                                          'PackageType' => 'PK',
                                                                                          'ParcelDepth' => '0'
                                                                                        }
                                                                    },
                                               'Shipper' => {
                                                              'Communication' => {
                                                                                   'email' => 'EMail',
                                                                                   'mobile' => 'PhoneCell',
                                                                                   'phone' => 'Phone',
                                                                                   'contactPerson' => 'FullName'
                                                                                 },
                                                              'Address' => {
                                                                             'city' => 'City',
                                                                             'streetNumber' => 'HouseNumber',
                                                                             'Origin' => {
                                                                                           'countryISOCode' => 'Code2'
                                                                                         },
                                                                             'Zip' => {
                                                                                        'other' => 'Zipcode'
                                                                                      },
                                                                             'streetName' => 'StreetOnly'
                                                                           },
                                                              'Company' => {
                                                                             'Company' => {
                                                                                            'name1' => 'Company',
                                                                                          },
                                                                           }
                                                            },
                                               'Receiver' => {
                                                               'Communication' => {
                                                                                    'email' => 'EMail',
                                                                                    'mobile' => 'PhoneCell',
                                                                                    'phone' => 'Phone',
                                                                                  },
                                                               'Address' => {
                                                                              'city' => 'City',
                                                                              'streetNumber' => 'HouseNumber',
                                                                              'Origin' => {
                                                                                            'countryISOCode' => 'Code2'
                                                                                          },
                                                                              'Zip' => {
                                                                                         'other' => 'Zipcode'
                                                                                       },
                                                                              'streetName' => 'StreetOnly'
                                                                            },
                                                               'Company' => {
                                                                              'Company' => {
                                                                                            'name1' => 'Company',
                                                                                            'name2' => 'FullName',
                                                                                          },
                                                                            }
                                                             }
                                             },
                               'SequenceNumber' => 'Alias'
                             },
          'header' => {
                        'signature' => 'WebservicePassword',
                        'user' => 'WebserviceUsername',
                        'type' => '0'
                      }
        },
        'Check full hash construction of the createShipmentDD'
    );
    return $testValue;
}

sub TestPackInternational {
    my ($DHLBuilder) = @_;
    my $testClass = MockObjectPM();
    note((caller(0))[3]);
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Economy', 'ServiceGroup' => 'PackInternational'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Premium', 'ServiceGroup' => 'PackInternational'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Seapacket', 'ServiceGroup' => 'PackInternational'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'CoilWithoutHelp', 'ServiceGroup' => 'PackInternational'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'AmountInternational', 'ServiceGroup' => 'PackInternational'});

    my $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();

    my $returnValue = $DHLBuilder->_assemblePackInternacional($SLInfo);

    is_deeply($returnValue, {
        'ServiceGroupBusinessPackInternational' => {
        'Economy' => 1,
        'Premium' => 1,
        'Seapacket' => 1,
        'CoilWithoutHelp' => 1,
        'AmountInternational' => 1
    }}, 'Checks if returned hash has the correct format (All bool options true)');


    $testClass = MockObjectPM();
    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();

    $returnValue = $DHLBuilder->_assemblePackInternacional($SLInfo);

    is_deeply($returnValue, {}, 'No Pack Internacional Options (should be and empty hash ref)');

    $testClass = MockObjectPM();

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'CoilWithoutHelp', 'ServiceGroup' => 'PackInternational'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'AmountInternational', 'ServiceGroup' => 'PackInternational'});

    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();

    $returnValue = $DHLBuilder->_assemblePackInternacional($SLInfo);
    is_deeply($returnValue, {
        'ServiceGroupBusinessPackInternational' => {
        'CoilWithoutHelp' => 1,
        'AmountInternational' => 1
    }}, 'Checks if returned hash has the correct format (CoilWithoutHelp and AmountInternational true)');
}

sub TestGroupOther {
    my ($DHLBuilder) = @_;
    my $testClass = MockObjectPM();
    note((caller(0))[3]);

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'DirectInjection', 'ServiceGroup' => 'GroupOther'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Bypass', 'ServiceGroup' => 'GroupOther'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'HigherInsuranceALTA', 'ServiceGroup' => 'GroupOther'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'HigherInsuranceALTB', 'ServiceGroup' => 'GroupOther'});

    my $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    my $ProdInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $testClass );
    $ProdInfo->loadAttributes();

    my $returnValue = $DHLBuilder->_assembleGroupOther($SLInfo, $ProdInfo);

    is_deeply($returnValue, {
        'ServiceGroupOther' => {
        'DirectInjection' => 1,
        'Bypass' => 1,
        'HigherInsurance' => {
            'InsuranceAmount' => 'LineItemsSum',
            'InsuranceCurrency' => 'EUR'
        }
    }}, 'Checks if returned hash has the correct format (All bool options true)');

    $testClass = MockObjectPM();

    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    $ProdInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $testClass );
    $ProdInfo->loadAttributes();

    $returnValue = $DHLBuilder->_assembleGroupOther($SLInfo, $ProdInfo);

    is_deeply($returnValue, {}, 'No Group Other Options (should be and empty hash ref)');

    $testClass = MockObjectPM();

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'DirectInjection', 'ServiceGroup' => 'GroupOther'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'HigherInsuranceALTA', 'ServiceGroup' => 'GroupOther'});

    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    $ProdInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $testClass );
    $ProdInfo->loadAttributes();

    $returnValue = $DHLBuilder->_assembleGroupOther($SLInfo, $ProdInfo);
    is_deeply($returnValue, {
        'ServiceGroupOther' => {
        'DirectInjection' => 1,
        'HigherInsurance' => {
            'InsuranceAmount' => 'LineItemsSum',
            'InsuranceCurrency' => 'EUR'
        }
    }}, 'Checks if returned hash has the correct format (DirectInjection true and HigherInsurance filled )');
}

sub TestGroupIdent {

    my ($DHLBuilder) = @_;
    my $testClass = MockObjectPM();
    note((caller(0))[3]);

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Personally', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'ReturnReceipt', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'ProofOfDelivery', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});

    my $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    my $returnValue = $DHLBuilder->_assembleGroupIdent($SLInfo);

    is_deeply($returnValue, {
        'ShipmentServiceGroupIdent' => {
        'Personally' => 1,
        'ReturnReceipt' => 1,
        'ProofOfDelivery' => 1,
    }}, 'Checks if returned hash has the correct format (All bool options true)');

    $testClass = MockObjectPM();
    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    $returnValue = $DHLBuilder->_assembleGroupIdent($SLInfo);
    is_deeply($returnValue, {}, 'No Group Ident Options (should be and empty hash ref)');

    $testClass = MockObjectPM();

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'ReturnReceipt', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    $returnValue = $DHLBuilder->_assembleGroupIdent($SLInfo);
    is_deeply($returnValue, {
        'ShipmentServiceGroupIdent' => {
        'ReturnReceipt' => 1,
    }}, 'Checks if returned hash has the correct format (ReturnReceipt is true)');
}

sub TestGroupDHLPaket {

    my ($DHLBuilder) = @_;
    my $testClass = MockObjectPM();
    note((caller(0))[3]);

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Multipack', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'ParticularDelivery', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'RegioPacket', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});

    my $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    my $returnValue = $DHLBuilder->_assemblerDHLPaket($SLInfo);

    is_deeply($returnValue, {
        'ServiceGroupDHLPaket' => {
        'Multipack' => 1,
        'RegioPacket' => 1,
        'ParticularDelivery' => 1,
    }}, 'Checks if returned hash has the correct format (All bool options true)');


    $testClass = MockObjectPM();
    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    $returnValue = $DHLBuilder->_assemblerDHLPaket($SLInfo);
    is_deeply($returnValue, {}, 'No Group Ident Options (should be and empty hash ref)');

    $testClass = MockObjectPM();

    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'Multipack', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $testClass->insertChild('DHLIntrashipService', {'ServiceName' => 'RegioPacket', 'ServiceGroup' => 'ShipmentServiceGroupIdent'});
    $SLInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $testClass );
    $SLInfo->loadAttributes();
    $returnValue = $DHLBuilder->_assemblerDHLPaket($SLInfo);
    is_deeply($returnValue, {
        'ServiceGroupDHLPaket' => {
        'RegioPacket' => 1,
        'Multipack' => 1,
    }}, 'Checks if returned hash has the correct format (ReturnReceipt is true)');
}

RunScript( Sub => \&Main );

## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitPrivateSubroutineUse)
1;